
; ---------------------------------------------------------------------------
; NewTestBed.asm
; ---------------------------------------------------------------------
; Reads a file containing a Console Screen Format into an array of
; bytes, converts it into a
; structure CHAR_INFO, and INVOKEs the API: WriteConsoleOutput()
; to display it all at once . Waits a key and terminates the program.
;----------------------------------------------------------------------
; Input file: Choose with an Open File Dialog a POP file to display
;----------------------------------------------------------------------
; Author: frktons @ MASM32 forum
; Date: 30/oct/2010. 
;---------------------------------------------------------------------- 
; In this version the sizes of the screen and the console are read 
; from the first 4 bytes, indicating RowInitial, RowFinal, ColInitial,
; ColFinal.
;----------------------------------------------------------------------
; Frank
;----------------------------------------------------------------------------


include \masm32\include\masm32rt.inc

GetConsole PROTO :DWORD
DisplayFmt PROTO :DWORD

;----------------------------------------------------------------------------
; Numeric constants used by the program
;----------------------------------------------------------------------------

Zero          EQU 0                  ;initial position on the buffer
Four          EQU 4                  ;number of initial bytes to read

MaxRows       EQU 32                 ;Max Rows of Console
MaxCols       EQU 90                 ;Max Cols of Console

MaxSize       EQU MaxRows*MaxCols*4  ;Max buffer size


;----------------------------------------------------------------------------
; Initialized data
;----------------------------------------------------------------------------

.data


    ConTitle  db "  ----------     **  New Testbed General Screen File "   
              db "Display Tool  **      ----------", 0   
    inpbyte   DWORD  1


    CaptionText   BYTE  "Open a Screen File to Display",0

    szFilter      BYTE  "POP files *.pop",0,"NewTestBed.pop",0,0
    
; ---------------------------------------------------------------------------    
;                        "SCD files *.scd",0,"*.scd",0,
;                        "All files *.*",0,"*.*",0,0
;----------------------------------------------------------------------------                           
          
;----------------------------------------------------------------------------
; Uninitialized data
;----------------------------------------------------------------------------         

.data?

    ConsoleScreen CHAR_INFO MaxSize dup (<>)
    
    RowInitial    db   ?    
    RowFinal      db   ?    
    ColInitial    db   ?    
    ColFinal      db   ?    

    R1            dw   ?
    R2            dw   ?
    C1            dw   ?
    C2            dw   ?
    
    FileBuffer    db   MaxSize  dup (?)
    BytesRead     dd   ?

    BufFileBytes  dd   ?          ;buffer size to read the POP file

    BufRows       dw   ?          ;buffer height (rows)
    BufCols       dw   ?          ;buffer width (columns)
    BufChars      dd   ?          ;number of chars in the buffer


    PtrFileName   DWORD ?   

    hFile     HANDLE ? 
    cci       CONSOLE_CURSOR_INFO <>        


    wHnd      HANDLE ? 
    rHnd      HANDLE ?

    windowSize SMALL_RECT <>
    bufferSize COORD      <>
    startPoint COORD      <>

    howmany dd ?
    buffer  INPUT_RECORD <>


.code

start: 

; ---------------------------------------------------------------------------
; The Main Proc shows the program flow.
; ---------------------------------------------------------------------------

Main PROC


    CALL   ConsoleSize 

    mov PtrFileName, OpenFileDlg (0,0, addr CaptionText, addr szFilter) 

    cmp BYTE PTR [eax], 0 ; test if 1st byte is zero [No selection made]

    je  Finish

    INVOKE GetConsole, DWORD PTR PtrFileName

    CALL   InitProc

    CALL   ConvertArea

    INVOKE DisplayFmt, offset ConsoleScreen
       
    CALL   AnyKey

    CALL   ShowTheCursor

Finish: 

   INVOKE ExitProcess,0

;  ret

Main ENDP

 
; ---------------------------------------------------------------------------
; The size, title and buffer of the Console are set.
; Max Console Size is set to 40 Rows and 132 Cols.
; ---------------------------------------------------------------------------

ConsoleSize PROC

    INVOKE GetStdHandle, STD_INPUT_HANDLE
    mov rHnd,eax

    INVOKE GetStdHandle, STD_OUTPUT_HANDLE
    mov wHnd,eax 

    INVOKE SetConsoleTitle, ADDR ConTitle
  
    CALL   HideTheCursor

    mov windowSize.Left, Zero
    mov windowSize.Right, (MaxCols -1)
    mov windowSize.Top, Zero
    mov windowSize.Bottom, (MaxRows -1)

    
    mov bufferSize.x, MaxCols
    mov bufferSize.y, MaxRows

    INVOKE SetConsoleWindowInfo, wHnd, TRUE, ADDR windowSize

    INVOKE SetConsoleScreenBufferSize, wHnd, DWORD PTR bufferSize

    ret

ConsoleSize ENDP

; ---------------------------------------------------------------------------
; Reads the Screen File.
; ---------------------------------------------------------------------------
; 1] The initial 4 bytes contain:
;
;    RowInitial, RowFinal, ColInitial, ColFinal of the Screen File to display
;
; 2] The remaining bytes contain the pairs of Characters/Attributes of
;
;    the Screen File in DOS style.
; ---------------------------------------------------------------------------
 

GetConsole PROC szFmtName:DWORD 

    mov hFile, fopen(szFmtName)

    mov ebx, fread(hFile, ADDR RowInitial, Four)

    movzx eax, RowFinal
    mov   R2,  ax
    inc   eax
    sub   al,  RowInitial

    mov   BufRows, ax

    movzx ebx, ColFinal
    mov   C2,  bx    
    inc   ebx
    sub   bl,  ColInitial

    mov   BufCols, bx

    imul  eax, ebx
    shl   eax, 1
    mov   BufFileBytes, eax  

    movzx ax, RowInitial  
    mov   R1, ax

    dec   R1
    dec   R2
    movzx ax, ColInitial
    mov   C1, ax 
    dec   C1   
    dec   C2

    mov   ebx, fread(hFile, ADDR FileBuffer, BufFileBytes)

    mov   BytesRead, ebx
    shr   ebx, 1
    mov   BufChars, ebx
    
    fclose hFile

    ret 
    
GetConsole ENDP

;----------------------------------------------------------------------------
; The Cursor is made visible
;----------------------------------------------------------------------------

ShowTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,TRUE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
ShowTheCursor ENDP

;----------------------------------------------------------------------------
; The Cursor is made invisible
;----------------------------------------------------------------------------
    


HideTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,FALSE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
HideTheCursor ENDP

;----------------------------------------------------------------------------
; After the Screen File has been read, the parameters to display it are set
;----------------------------------------------------------------------------

InitProc PROC

 
    mov ax, C1
    mov windowSize.Left, ax
    mov ax, C2
    mov windowSize.Right, ax
    mov ax, R1
    mov windowSize.Top, ax
    mov ax, R2
    mov windowSize.Bottom, ax
   
    mov ax, BufRows
    mov bx, BufCols
    
    mov bufferSize.x, bx
    mov bufferSize.y, ax

    mov startPoint.x, Zero
    mov startPoint.y, Zero


    ret

InitProc ENDP

;----------------------------------------------------------------------------
; The Screen File is displayed on the Console 
;----------------------------------------------------------------------------

DisplayFmt PROC szFmtName:DWORD 

    INVOKE WriteConsoleOutput, wHnd, szFmtName, DWORD PTR bufferSize, 
           DWORD PTR startPoint, offset windowSize 

    ret

DisplayFmt ENDP

;----------------------------------------------------------------------------
; The program waits for a key pressed
;----------------------------------------------------------------------------
; Returns: key code in buffer.KeyEvent.wVirtualKeyCode WORD size 
; -------------------------------------------------------------------------

AnyKey PROC

again:  

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz again

    cmp buffer.KeyEvent.bKeyDown,0
    jz again

    ret 

AnyKey ENDP


; -------------------------------------------------------------------------
; The data read from the file is converted into CHAR_INFO style. 
; ConsoleBuffer is filled with the content of FileBuffer, padding a zero
; after each byte making it Windows Console API displayable. 
; -------------------------------------------------------------------------

ConvertArea PROC

    lea  esi, FileBuffer
    lea  edi, ConsoleScreen

    mov  ecx, BufChars
    xor  eax, eax

    xor  ebx, ebx
    
NextChar:

    mov  bx , WORD PTR [esi]
    mov  ah , bh
    shl  eax, 8
    mov  al , bl

    mov  [edi], eax
    add  esi, 2
    add  edi, 4

    dec  ecx

    jnz  NextChar

    ret 
    
ConvertArea ENDP

;----------------------------------------------------------------------------
; End of program
;----------------------------------------------------------------------------

end start